const modifiers      = require('./index');
const utils          = require('../utils');
const defaultOptions = require('../defaultOptions');

const transform = {
  defaults: {
    enabled    : true,
    endOnly    : false,
    transformation: 0.5,
    elementRect: null,
  },

  shouldDo: function (interactable, actionName, preEnd, requireEndOnly) {
    const transformOptions = interactable.options[actionName].transform;

    return (transformOptions && transformOptions.enabled
            && (preEnd || !transformOptions.endOnly)
            && (!requireEndOnly || transformOptions.endOnly));
  },

  setOffset: function (interaction, interactable, element, rect, startOffset) {
    /*
    const elementRect = interactable.options[interaction.prepared.name].transform.elementRect;
    const offset = {};

    if (rect && elementRect) {
      offset.left = startOffset.left - (rect.width  * elementRect.left);
      offset.top  = startOffset.top  - (rect.height * elementRect.top);

      offset.right  = startOffset.right  - (rect.width  * (1 - elementRect.right));
      offset.bottom = startOffset.bottom - (rect.height * (1 - elementRect.bottom));
    }
    else {
      offset.left = offset.top = offset.right = offset.bottom = 0;
    }

    return offset;
   */
  },

  set: function (pageCoords, interaction, status) {
    const target    = interaction.target;
    const transformOptions  = target && target.options[interaction.prepared.name].transform;
    let transformation = transformOptions && transformOptions.transformation;

    if (!transformation) {
      return status;
    }

    const page = status.useStatusXY
      ? { x: status.x, y: status.y }
      : utils.extend({}, pageCoords);

    status.dx = 0;
    status.dy = 0;
    status.locked = false;

    let rect;
    let transformedX = page.x * transformation;
    let transformedY = page.y * transformation;

    status.dx = transformedX - page.x;
    status.dy = transformedY - page.y;

    status.changed = status.transformedX !== transformedX || status.transformedY !== transformedY;
    status.locked = !!(status.dx || status.dy);

    status.transformedX = transformedX;
    status.transformedY = transformedY;

    return status;
  },

  reset: function (status) {
    status.dx = status.dy = 0;
    status.modifiedX = status.modifiedY = NaN;
    status.locked = false;
    status.changed = true;

    return status;
  },

  modifyCoords: function (page, client, interactable, status, actionName, phase) {
    const options = interactable.options[actionName].transform;
    const elementRect = options && options.elementRect;

    if (options && options.enabled
        && !(phase === 'start' && elementRect && status.locked)) {

      if (status.locked) {
        page.x += status.dx;
        page.y += status.dy;
        client.x += status.dx;
        client.y += status.dy;

        return {
          dx: status.dx,
          dy: status.dy,
        };
      }
    }
  },
};

modifiers.transform = transform;
modifiers.names.push('transform');

defaultOptions.perAction.transform = transform.defaults;

module.exports = transform;
